/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../openal-soft/utils/alsoft-config/mainwindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainWindow",
        "cancelCloseAction",
        "",
        "saveCurrentConfig",
        "saveConfigAsFile",
        "loadConfigFromFile",
        "showAboutPage",
        "enableApplyButton",
        "updateResamplerLabel",
        "num",
        "updatePeriodSizeEdit",
        "size",
        "updatePeriodSizeSlider",
        "updatePeriodCountEdit",
        "count",
        "updatePeriodCountSlider",
        "selectQuadDecoderFile",
        "select51DecoderFile",
        "select61DecoderFile",
        "select71DecoderFile",
        "select3D71DecoderFile",
        "updateJackBufferSizeEdit",
        "updateJackBufferSizeSlider",
        "updateHrtfModeLabel",
        "addHrtfFile",
        "removeHrtfFile",
        "updateHrtfRemoveButton",
        "showEnabledBackendMenu",
        "QPoint",
        "pt",
        "showDisabledBackendMenu",
        "selectOSSPlayback",
        "selectOSSCapture",
        "selectSolarisPlayback",
        "selectWaveOutput"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'cancelCloseAction'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveCurrentConfig'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveConfigAsFile'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'loadConfigFromFile'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showAboutPage'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'enableApplyButton'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateResamplerLabel'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'updatePeriodSizeEdit'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'updatePeriodSizeSlider'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updatePeriodCountEdit'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'updatePeriodCountSlider'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectQuadDecoderFile'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'select51DecoderFile'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'select61DecoderFile'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'select71DecoderFile'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'select3D71DecoderFile'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateJackBufferSizeEdit'
        QtMocHelpers::SlotData<void(int)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'updateJackBufferSizeSlider'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateHrtfModeLabel'
        QtMocHelpers::SlotData<void(int)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'addHrtfFile'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'removeHrtfFile'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateHrtfRemoveButton'
        QtMocHelpers::SlotData<void() const>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showEnabledBackendMenu'
        QtMocHelpers::SlotData<void(QPoint)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'showDisabledBackendMenu'
        QtMocHelpers::SlotData<void(QPoint)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'selectOSSPlayback'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectOSSCapture'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectSolarisPlayback'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectWaveOutput'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10MainWindowE_t>.metaTypes,
    nullptr
} };

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->cancelCloseAction(); break;
        case 1: _t->saveCurrentConfig(); break;
        case 2: _t->saveConfigAsFile(); break;
        case 3: _t->loadConfigFromFile(); break;
        case 4: _t->showAboutPage(); break;
        case 5: _t->enableApplyButton(); break;
        case 6: _t->updateResamplerLabel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->updatePeriodSizeEdit((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->updatePeriodSizeSlider(); break;
        case 9: _t->updatePeriodCountEdit((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->updatePeriodCountSlider(); break;
        case 11: _t->selectQuadDecoderFile(); break;
        case 12: _t->select51DecoderFile(); break;
        case 13: _t->select61DecoderFile(); break;
        case 14: _t->select71DecoderFile(); break;
        case 15: _t->select3D71DecoderFile(); break;
        case 16: _t->updateJackBufferSizeEdit((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->updateJackBufferSizeSlider(); break;
        case 18: _t->updateHrtfModeLabel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->addHrtfFile(); break;
        case 20: _t->removeHrtfFile(); break;
        case 21: _t->updateHrtfRemoveButton(); break;
        case 22: _t->showEnabledBackendMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 23: _t->showDisabledBackendMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 24: _t->selectOSSPlayback(); break;
        case 25: _t->selectOSSCapture(); break;
        case 26: _t->selectSolarisPlayback(); break;
        case 27: _t->selectWaveOutput(); break;
        default: ;
        }
    }
}

const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 28;
    }
    return _id;
}
QT_WARNING_POP
